@@ -20,6 +20,7 @@ import sys
 
 from keystoneauth1 import exceptions as ks_exc
 from osc_lib.command import command
+from osc_lib import exceptions
 from osc_lib import utils
 import six
 
@@ -43,7 +44,9 @@ class AddUserToGroup(command.Command):
         parser.add_argument(
             'user',
             metavar='<user>',
-            help=_('User to add to <group> (name or ID)'),
+            nargs='+',
+            help=_('User(s) to add to <group> (name or ID) '
+                   '(repeat option to add multiple users)'),
         )
         common.add_group_domain_option_to_parser(parser)
         common.add_user_domain_option_to_parser(parser)
@@ -52,27 +55,34 @@ class AddUserToGroup(command.Command):
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
 
-        user_id = common.find_user(identity_client,
-                                   parsed_args.user,
-                                   parsed_args.user_domain).id
         group_id = common.find_group(identity_client,
                                      parsed_args.group,
                                      parsed_args.group_domain).id
 
-        try:
-            identity_client.users.add_to_group(user_id, group_id)
-        except Exception:
-            msg = _("%(user)s not added to group %(group)s\n") % {
-                'user': parsed_args.user,
+        result = 0
+        for i in parsed_args.user:
+            try:
+                user_id = common.find_user(identity_client,
+                                           i,
+                                           parsed_args.user_domain).id
+                identity_client.users.add_to_group(user_id, group_id)
+            except Exception as e:
+                result += 1
+                msg = _("%(user)s not added to group %(group)s: %(e)s") % {
+                    'user': i,
+                    'group': parsed_args.group,
+                    'e': e,
+                }
+                LOG.error(msg)
+        if result > 0:
+            total = len(parsed_args.user)
+            msg = (_("%(result)s of %(total)s users not added to group "
+                   "%(group)s.")) % {
+                'result': result,
+                'total': total,
                 'group': parsed_args.group,
             }
-            sys.stderr.write(msg)
-        else:
-            msg = _("%(user)s added to group %(group)s\n") % {
-                'user': parsed_args.user,
-                'group': parsed_args.group,
-            }
-            sys.stdout.write(msg)
+            raise exceptions.CommandError(msg)
 
 
 class CheckUserInGroup(command.Command):
@@ -106,12 +116,15 @@ class CheckUserInGroup(command.Command):
 
         try:
             identity_client.users.check_in_group(user_id, group_id)
-        except Exception:
-            msg = _("%(user)s not in group %(group)s\n") % {
-                'user': parsed_args.user,
-                'group': parsed_args.group,
-            }
-            sys.stderr.write(msg)
+        except ks_exc.http.HTTPClientError as e:
+            if e.http_status == 403 or e.http_status == 404:
+                msg = _("%(user)s not in group %(group)s\n") % {
+                    'user': parsed_args.user,
+                    'group': parsed_args.group,
+                }
+                sys.stderr.write(msg)
+            else:
+                raise e
         else:
             msg = _("%(user)s in group %(group)s\n") % {
                 'user': parsed_args.user,
@@ -194,11 +207,24 @@ class DeleteGroup(command.Command):
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
 
+        errors = 0
         for group in parsed_args.groups:
-            group_obj = common.find_group(identity_client,
-                                          group,
-                                          parsed_args.domain)
-            identity_client.groups.delete(group_obj.id)
+            try:
+                group_obj = common.find_group(identity_client,
+                                              group,
+                                              parsed_args.domain)
+                identity_client.groups.delete(group_obj.id)
+            except Exception as e:
+                errors += 1
+                LOG.error(_("Failed to delete group with "
+                          "name or ID '%(group)s': %(e)s"),
+                          {'group': group, 'e': e})
+
+        if errors > 0:
+            total = len(parsed_args.groups)
+            msg = (_("%(errors)s of %(total)s groups failed "
+                   "to delete.") % {'errors': errors, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListGroup(command.Lister):
@@ -274,7 +300,9 @@ class RemoveUserFromGroup(command.Command):
         parser.add_argument(
             'user',
             metavar='<user>',
-            help=_('User to remove from <group> (name or ID)'),
+            nargs='+',
+            help=_('User(s) to remove from <group> (name or ID) '
+                   '(repeat option to remove multiple users)'),
         )
         common.add_group_domain_option_to_parser(parser)
         common.add_user_domain_option_to_parser(parser)
@@ -283,27 +311,34 @@ class RemoveUserFromGroup(command.Command):
     def take_action(self, parsed_args):
         identity_client = self.app.client_manager.identity
 
-        user_id = common.find_user(identity_client,
-                                   parsed_args.user,
-                                   parsed_args.user_domain).id
         group_id = common.find_group(identity_client,
                                      parsed_args.group,
                                      parsed_args.group_domain).id
 
-        try:
-            identity_client.users.remove_from_group(user_id, group_id)
-        except Exception:
-            msg = _("%(user)s not removed from group %(group)s\n") % {
-                'user': parsed_args.user,
+        result = 0
+        for i in parsed_args.user:
+            try:
+                user_id = common.find_user(identity_client,
+                                           i,
+                                           parsed_args.user_domain).id
+                identity_client.users.remove_from_group(user_id, group_id)
+            except Exception as e:
+                result += 1
+                msg = _("%(user)s not removed from group %(group)s: %(e)s") % {
+                    'user': i,
+                    'group': parsed_args.group,
+                    'e': e,
+                }
+                LOG.error(msg)
+        if result > 0:
+            total = len(parsed_args.user)
+            msg = (_("%(result)s of %(total)s users not removed from group "
+                   "%(group)s.")) % {
+                'result': result,
+                'total': total,
                 'group': parsed_args.group,
             }
-            sys.stderr.write(msg)
-        else:
-            msg = _("%(user)s removed from group %(group)s\n") % {
-                'user': parsed_args.user,
-                'group': parsed_args.group,
-            }
-            sys.stdout.write(msg)
+            raise exceptions.CommandError(msg)
 
 
 class SetGroup(command.Command):
