@@ -36,8 +36,11 @@ from openstackclient.api import utils as api_utils
 from openstackclient.i18n import _
 
 
+CONTAINER_CHOICES = ["ami", "ari", "aki", "bare", "docker", "ova", "ovf"]
 DEFAULT_CONTAINER_FORMAT = 'bare'
 DEFAULT_DISK_FORMAT = 'raw'
+DISK_CHOICES = ["ami", "ari", "aki", "vhd", "vmdk", "raw", "qcow2", "vhdx",
+                "vdi", "iso", "ploop"]
 
 
 LOG = logging.getLogger(__name__)
@@ -82,15 +85,20 @@ class CreateImage(command.ShowOne):
             "--container-format",
             default=DEFAULT_CONTAINER_FORMAT,
             metavar="<container-format>",
-            help=_("Image container format "
-                   "(default: %s)") % DEFAULT_CONTAINER_FORMAT,
+            choices=CONTAINER_CHOICES,
+            help=(_("Image container format. "
+                    "The supported options are: %(option_list)s. "
+                    "The default format is: %(default_opt)s") %
+                  {'option_list': ', '.join(CONTAINER_CHOICES),
+                   'default_opt': DEFAULT_CONTAINER_FORMAT})
         )
         parser.add_argument(
             "--disk-format",
             default=DEFAULT_DISK_FORMAT,
             metavar="<disk-format>",
-            help=_("Image disk format "
-                   "(default: %s)") % DEFAULT_DISK_FORMAT,
+            choices=DISK_CHOICES,
+            help=_("Image disk format. The supported options are: %s. "
+                   "The default format is: raw") % ', '.join(DISK_CHOICES)
         )
         parser.add_argument(
             "--size",
@@ -120,12 +128,13 @@ class CreateImage(command.ShowOne):
             metavar="<image-url>",
             help=_("Copy image from the data store (similar to --location)"),
         )
-        parser.add_argument(
+        source_group = parser.add_mutually_exclusive_group()
+        source_group.add_argument(
             "--file",
             metavar="<file>",
             help=_("Upload image from local file"),
         )
-        parser.add_argument(
+        source_group.add_argument(
             "--volume",
             metavar="<volume>",
             help=_("Create image from a volume"),
@@ -350,8 +359,9 @@ class ListImage(command.Lister):
         parser.add_argument(
             '--sort',
             metavar="<key>[:<direction>]",
+            default='name:asc',
             help=_("Sort output by selected keys and directions(asc or desc) "
-                   "(default: asc), multiple keys and directions can be "
+                   "(default: name:asc), multiple keys and directions can be "
                    "specified separated by comma"),
         )
         return parser
@@ -494,22 +504,19 @@ class SetImage(command.Command):
             type=int,
             help=_("Minimum RAM size needed to boot image, in megabytes"),
         )
-        container_choices = ["ami", "ari", "aki", "bare", "ovf"]
         parser.add_argument(
             "--container-format",
             metavar="<container-format>",
-            help=_("Container format of image. Acceptable formats: %s") %
-            container_choices,
-            choices=container_choices
+            choices=CONTAINER_CHOICES,
+            help=_("Image container format. The supported options are: %s") %
+            ', '.join(CONTAINER_CHOICES)
         )
-        disk_choices = ["ami", "ari", "aki", "vhd", "vmdk", "raw", "qcow2",
-                        "vdi", "iso"]
         parser.add_argument(
             "--disk-format",
             metavar="<disk-format>",
-            help=_("Disk format of image. Acceptable formats: %s") %
-            disk_choices,
-            choices=disk_choices
+            choices=DISK_CHOICES,
+            help=_("Image disk format. The supported options are: %s.") %
+            ', '.join(DISK_CHOICES)
         )
         parser.add_argument(
             "--size",
