@@ -51,7 +51,6 @@ _formatters = {
     'extra_dhcp_opts': utils.format_list_of_dicts,
     'fixed_ips': utils.format_list_of_dicts,
     'security_group_ids': utils.format_list,
-    'security_groups': utils.format_list,
 }
 
 
@@ -64,7 +63,6 @@ def _get_columns(item):
         'binding:vnic_type': 'binding_vnic_type',
         'is_admin_state_up': 'admin_state_up',
         'is_port_security_enabled': 'port_security_enabled',
-        'security_group_ids': 'security_groups',
         'tenant_id': 'project_id',
     }
     return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)
@@ -116,8 +114,6 @@ def _get_attrs(client_manager, parsed_args):
         ))
     if parsed_args.description is not None:
         attrs['description'] = parsed_args.description
-    if parsed_args.fixed_ip is not None:
-        attrs['fixed_ips'] = parsed_args.fixed_ip
     if parsed_args.device:
         attrs['device_id'] = parsed_args.device
     if parsed_args.device_owner is not None:
@@ -126,12 +122,12 @@ def _get_attrs(client_manager, parsed_args):
         attrs['admin_state_up'] = True
     if parsed_args.disable:
         attrs['admin_state_up'] = False
-    if parsed_args.binding_profile is not None:
-        attrs['binding:profile'] = parsed_args.binding_profile
     if parsed_args.vnic_type is not None:
         attrs['binding:vnic_type'] = parsed_args.vnic_type
     if parsed_args.host:
         attrs['binding:host_id'] = parsed_args.host
+    if parsed_args.mac_address is not None:
+        attrs['mac_address'] = parsed_args.mac_address
 
     if parsed_args.dns_name is not None:
         attrs['dns_name'] = parsed_args.dns_name
@@ -140,8 +136,6 @@ def _get_attrs(client_manager, parsed_args):
         attrs['name'] = str(parsed_args.name)
     # The remaining options do not support 'port set' command, so they require
     # additional check
-    if 'mac_address' in parsed_args and parsed_args.mac_address is not None:
-        attrs['mac_address'] = parsed_args.mac_address
     if 'network' in parsed_args and parsed_args.network is not None:
         attrs['network_id'] = parsed_args.network
     if 'project' in parsed_args and parsed_args.project is not None:
@@ -161,6 +155,13 @@ def _get_attrs(client_manager, parsed_args):
     if parsed_args.enable_port_security:
         attrs['port_security_enabled'] = True
 
+    if 'no_qos_policy' in parsed_args and parsed_args.no_qos_policy:
+        attrs['qos_policy_id'] = None
+
+    if parsed_args.qos_policy:
+        attrs['qos_policy_id'] = client_manager.network.find_qos_policy(
+            parsed_args.qos_policy, ignore_missing=False).id
+
     return attrs
 
 
@@ -194,6 +195,29 @@ def _prepare_fixed_ips(client_manager, parsed_args):
         parsed_args.fixed_ip = ips
 
 
+def _prepare_filter_fixed_ips(client_manager, parsed_args):
+    """Fix and properly format fixed_ip option for filtering.
+
+    Appropriately convert any subnet names to their respective ids.
+    Convert fixed_ips in parsed args to be in valid list format for filter:
+    ['subnet_id=foo'].
+    """
+    client = client_manager.network
+    ips = []
+
+    for ip_spec in parsed_args.fixed_ip:
+        if 'subnet' in ip_spec:
+            subnet_name_id = ip_spec['subnet']
+            if subnet_name_id:
+                _subnet = client.find_subnet(subnet_name_id,
+                                             ignore_missing=False)
+                ips.append('subnet_id=%s' % _subnet.id)
+
+        if 'ip-address' in ip_spec:
+            ips.append('ip_address=%s' % ip_spec['ip-address'])
+    return ips
+
+
 def _add_updatable_args(parser):
     parser.add_argument(
         '--description',
@@ -213,6 +237,11 @@ def _add_updatable_args(parser):
         metavar='<device-id>',
         help=argparse.SUPPRESS,
     )
+    parser.add_argument(
+        '--mac-address',
+        metavar='<mac-address>',
+        help=_("MAC address of this port (admin only)")
+    )
     parser.add_argument(
         '--device-owner',
         metavar='<device-owner>',
@@ -279,7 +308,7 @@ class CreatePort(command.ShowOne):
             metavar='subnet=<subnet>,ip-address=<ip-address>',
             action=parseractions.MultiKeyValueAction,
             optional_keys=['subnet', 'ip-address'],
-            help=_("Desired IP and/or subnet (name or ID) for this port: "
+            help=_("Desired IP and/or subnet for this port (name or ID): "
                    "subnet=<subnet>,ip-address=<ip-address> "
                    "(repeat option to set multiple fixed IP addresses)")
         )
@@ -303,11 +332,6 @@ class CreatePort(command.ShowOne):
             action='store_true',
             help=_("Disable port")
         )
-        parser.add_argument(
-            '--mac-address',
-            metavar='<mac-address>',
-            help=_("MAC address of this port")
-        )
         parser.add_argument(
             '--project',
             metavar='<project>',
@@ -320,13 +344,13 @@ class CreatePort(command.ShowOne):
             help=_("Name of this port")
         )
         # TODO(singhj): Add support for extended options:
-        # qos,dhcp
+        # dhcp
         secgroups = parser.add_mutually_exclusive_group()
         secgroups.add_argument(
             '--security-group',
             metavar='<security-group>',
             action='append',
-            dest='security_groups',
+            dest='security_group',
             help=_("Security group to associate with this port (name or ID) "
                    "(repeat option to set multiple security groups)")
         )
@@ -336,6 +360,11 @@ class CreatePort(command.ShowOne):
             action='store_true',
             help=_("Associate no security groups with this port")
         )
+        parser.add_argument(
+            '--qos-policy',
+            metavar='<qos-policy>',
+            help=_("Attach QoS policy to this port (name or ID)")
+        )
         port_security = parser.add_mutually_exclusive_group()
         port_security.add_argument(
             '--enable-port-security',
@@ -368,16 +397,27 @@ class CreatePort(command.ShowOne):
         _prepare_fixed_ips(self.app.client_manager, parsed_args)
         attrs = _get_attrs(self.app.client_manager, parsed_args)
 
-        if parsed_args.security_groups:
-            attrs['security_groups'] = [client.find_security_group(
-                                        sg, ignore_missing=False).id
-                                        for sg in parsed_args.security_groups]
-        if parsed_args.no_security_group:
-            attrs['security_groups'] = []
+        if parsed_args.binding_profile is not None:
+            attrs['binding:profile'] = parsed_args.binding_profile
+
+        if parsed_args.fixed_ip:
+            attrs['fixed_ips'] = parsed_args.fixed_ip
+
+        if parsed_args.security_group:
+            attrs['security_group_ids'] = [client.find_security_group(
+                                           sg, ignore_missing=False).id
+                                           for sg in
+                                           parsed_args.security_group]
+        elif parsed_args.no_security_group:
+            attrs['security_group_ids'] = []
+
         if parsed_args.allowed_address_pairs:
             attrs['allowed_address_pairs'] = (
                 _convert_address_pairs(parsed_args))
 
+        if parsed_args.qos_policy:
+            attrs['qos_policy_id'] = client.find_qos_policy(
+                parsed_args.qos_policy, ignore_missing=False).id
         obj = client.create_port(**attrs)
         display_columns, columns = _get_columns(obj)
         data = utils.get_item_properties(obj, columns, formatters=_formatters)
@@ -466,11 +506,19 @@ class ListPort(command.Lister):
             help=_("List ports according to their project (name or ID)")
         )
         identity_common.add_project_domain_option_to_parser(parser)
+        parser.add_argument(
+            '--fixed-ip',
+            metavar='subnet=<subnet>,ip-address=<ip-address>',
+            action=parseractions.MultiKeyValueAction,
+            optional_keys=['subnet', 'ip-address'],
+            help=_("Desired IP and/or subnet for filtering ports "
+                   "(name or ID): subnet=<subnet>,ip-address=<ip-address> "
+                   "(repeat option to set multiple fixed IP addresses)"),
+        )
         return parser
 
     def take_action(self, parsed_args):
         network_client = self.app.client_manager.network
-        compute_client = self.app.client_manager.compute
         identity_client = self.app.client_manager.identity
 
         columns = (
@@ -499,6 +547,7 @@ class ListPort(command.Lister):
                                                  ignore_missing=False)
             filters['device_id'] = _router.id
         if parsed_args.server:
+            compute_client = self.app.client_manager.compute
             server = utils.find_resource(compute_client.servers,
                                          parsed_args.server)
             filters['device_id'] = server.id
@@ -516,6 +565,9 @@ class ListPort(command.Lister):
             ).id
             filters['tenant_id'] = project_id
             filters['project_id'] = project_id
+        if parsed_args.fixed_ip:
+            filters['fixed_ips'] = _prepare_filter_fixed_ips(
+                self.app.client_manager, parsed_args)
 
         data = network_client.ports(**filters)
 
@@ -556,7 +608,7 @@ class SetPort(command.Command):
             metavar='subnet=<subnet>,ip-address=<ip-address>',
             action=parseractions.MultiKeyValueAction,
             optional_keys=['subnet', 'ip-address'],
-            help=_("Desired IP and/or subnet (name or ID) for this port: "
+            help=_("Desired IP and/or subnet for this port (name or ID): "
                    "subnet=<subnet>,ip-address=<ip-address> "
                    "(repeat option to set multiple fixed IP addresses)")
         )
@@ -582,6 +634,11 @@ class SetPort(command.Command):
                    "Specify both --binding-profile and --no-binding-profile "
                    "to overwrite the current binding:profile information.")
         )
+        parser.add_argument(
+            '--qos-policy',
+            metavar='<qos-policy>',
+            help=_("Attach QoS policy to this port (name or ID)")
+        )
         parser.add_argument(
             'port',
             metavar="<port>",
@@ -591,7 +648,7 @@ class SetPort(command.Command):
             '--security-group',
             metavar='<security-group>',
             action='append',
-            dest='security_groups',
+            dest='security_group',
             help=_("Security group to associate with this port (name or ID) "
                    "(repeat option to set multiple security groups)")
         )
@@ -638,51 +695,52 @@ class SetPort(command.Command):
         client = self.app.client_manager.network
 
         _prepare_fixed_ips(self.app.client_manager, parsed_args)
-        attrs = _get_attrs(self.app.client_manager, parsed_args)
         obj = client.find_port(parsed_args.port, ignore_missing=False)
-        if 'binding:profile' in attrs:
-            # Do not modify attrs if both binding_profile/no_binding given
-            if not parsed_args.no_binding_profile:
-                tmp_binding_profile = copy.deepcopy(obj.binding_profile)
-                tmp_binding_profile.update(attrs['binding:profile'])
-                attrs['binding:profile'] = tmp_binding_profile
-        elif parsed_args.no_binding_profile:
+        attrs = _get_attrs(self.app.client_manager, parsed_args)
+
+        if parsed_args.no_binding_profile:
             attrs['binding:profile'] = {}
-        if 'fixed_ips' in attrs:
-            # When user unsets the fixed_ips, obj.fixed_ips = [{}].
-            # Adding the obj.fixed_ips list to attrs['fixed_ips']
-            # would therefore add an empty dictionary, while we need
-            # to append the attrs['fixed_ips'] iff there is some info
-            # in the obj.fixed_ips. Therefore I have opted for this `for` loop
-            # Do not modify attrs if fixed_ip/no_fixed_ip given
-            if not parsed_args.no_fixed_ip:
-                attrs['fixed_ips'] += [ip for ip in obj.fixed_ips if ip]
-        elif parsed_args.no_fixed_ip:
+        if parsed_args.binding_profile:
+            if 'binding:profile' not in attrs:
+                attrs['binding:profile'] = copy.deepcopy(obj.binding_profile)
+            attrs['binding:profile'].update(parsed_args.binding_profile)
+
+        if parsed_args.no_fixed_ip:
             attrs['fixed_ips'] = []
-        if parsed_args.security_groups and parsed_args.no_security_group:
-            attrs['security_groups'] = [client.find_security_group(sg,
-                                        ignore_missing=False).id
-                                        for sg in parsed_args.security_groups]
-        elif parsed_args.security_groups:
-            attrs['security_groups'] = obj.security_groups
-            for sg in parsed_args.security_groups:
-                sg_id = client.find_security_group(sg, ignore_missing=False).id
-                attrs['security_groups'].append(sg_id)
-        elif parsed_args.no_security_group:
-            attrs['security_groups'] = []
+        if parsed_args.fixed_ip:
+            if 'fixed_ips' not in attrs:
+                # obj.fixed_ips = [{}] if no fixed IPs are set.
+                # Only append this to attrs['fixed_ips'] if actual fixed
+                # IPs are present to avoid adding an empty dict.
+                attrs['fixed_ips'] = [ip for ip in obj.fixed_ips if ip]
+            attrs['fixed_ips'].extend(parsed_args.fixed_ip)
 
-        if (parsed_args.allowed_address_pairs and
-                parsed_args.no_allowed_address_pair):
-            attrs['allowed_address_pairs'] = (
-                _convert_address_pairs(parsed_args))
+        if parsed_args.no_security_group:
+            attrs['security_group_ids'] = []
+        if parsed_args.security_group:
+            if 'security_group_ids' not in attrs:
+                # NOTE(dtroyer): Get existing security groups, iterate the
+                #                list to force a new list object to be
+                #                created and make sure the SDK Resource
+                #                marks the attribute 'dirty'.
+                attrs['security_group_ids'] = [
+                    id for id in obj.security_group_ids
+                ]
+            attrs['security_group_ids'].extend(
+                client.find_security_group(sg, ignore_missing=False).id
+                for sg in parsed_args.security_group
+            )
 
-        elif parsed_args.allowed_address_pairs:
-            attrs['allowed_address_pairs'] = (
-                [addr for addr in obj.allowed_address_pairs if addr] +
-                _convert_address_pairs(parsed_args))
-
-        elif parsed_args.no_allowed_address_pair:
+        if parsed_args.no_allowed_address_pair:
             attrs['allowed_address_pairs'] = []
+        if parsed_args.allowed_address_pairs:
+            if 'allowed_address_pairs' not in attrs:
+                attrs['allowed_address_pairs'] = (
+                    [addr for addr in obj.allowed_address_pairs if addr]
+                )
+            attrs['allowed_address_pairs'].extend(
+                _convert_address_pairs(parsed_args)
+            )
 
         client.update_port(obj, **attrs)
 
@@ -719,8 +777,8 @@ class UnsetPort(command.Command):
             metavar='subnet=<subnet>,ip-address=<ip-address>',
             action=parseractions.MultiKeyValueAction,
             optional_keys=['subnet', 'ip-address'],
-            help=_("Desired IP and/or subnet (name or ID) which should be "
-                   "removed from this port: subnet=<subnet>,"
+            help=_("Desired IP and/or subnet which should be "
+                   "removed from this port (name or ID): subnet=<subnet>,"
                    "ip-address=<ip-address> (repeat option to unset multiple "
                    "fixed IP addresses)"))
 
@@ -734,7 +792,7 @@ class UnsetPort(command.Command):
             '--security-group',
             metavar='<security-group>',
             action='append',
-            dest='security_groups',
+            dest='security_group_ids',
             help=_("Security group which should be removed this port (name "
                    "or ID) (repeat option to unset multiple security groups)")
         )
@@ -756,6 +814,12 @@ class UnsetPort(command.Command):
                    "[,mac-address=<mac-address>] (repeat option to set "
                    "multiple allowed-address pairs)")
         )
+        parser.add_argument(
+            '--qos-policy',
+            action='store_true',
+            default=False,
+            help=_("Remove the QoS policy attached to the port")
+        )
 
         return parser
 
@@ -767,7 +831,7 @@ class UnsetPort(command.Command):
         # Unset* classes
         tmp_fixed_ips = copy.deepcopy(obj.fixed_ips)
         tmp_binding_profile = copy.deepcopy(obj.binding_profile)
-        tmp_secgroups = copy.deepcopy(obj.security_groups)
+        tmp_secgroups = copy.deepcopy(obj.security_group_ids)
         tmp_addr_pairs = copy.deepcopy(obj.allowed_address_pairs)
         _prepare_fixed_ips(self.app.client_manager, parsed_args)
         attrs = {}
@@ -787,16 +851,16 @@ class UnsetPort(command.Command):
                 msg = _("Port does not contain binding-profile %s") % key
                 raise exceptions.CommandError(msg)
             attrs['binding:profile'] = tmp_binding_profile
-        if parsed_args.security_groups:
+        if parsed_args.security_group_ids:
             try:
-                for sg in parsed_args.security_groups:
+                for sg in parsed_args.security_group_ids:
                     sg_id = client.find_security_group(
                         sg, ignore_missing=False).id
                     tmp_secgroups.remove(sg_id)
             except ValueError:
                 msg = _("Port does not contain security group %s") % sg
                 raise exceptions.CommandError(msg)
-            attrs['security_groups'] = tmp_secgroups
+            attrs['security_group_ids'] = tmp_secgroups
         if parsed_args.allowed_address_pairs:
             try:
                 for addr in _convert_address_pairs(parsed_args):
@@ -805,6 +869,8 @@ class UnsetPort(command.Command):
                 msg = _("Port does not contain allowed-address-pair %s") % addr
                 raise exceptions.CommandError(msg)
             attrs['allowed_address_pairs'] = tmp_addr_pairs
+        if parsed_args.qos_policy:
+            attrs['qos_policy_id'] = None
 
         if attrs:
             client.update_port(obj, **attrs)
