@@ -41,11 +41,6 @@ class TestServer(compute_fakes.TestComputev2):
         self.flavors_mock = self.app.client_manager.compute.flavors
         self.flavors_mock.reset_mock()
 
-        # Get a shortcut to the compute client SecurityGroupManager Mock
-        self.security_groups_mock = \
-            self.app.client_manager.compute.security_groups
-        self.security_groups_mock.reset_mock()
-
         # Get a shortcut to the image client ImageManager Mock
         self.images_mock = self.app.client_manager.image.images
         self.images_mock.reset_mock()
@@ -96,38 +91,48 @@ class TestServerAddFixedIP(TestServer):
     def setUp(self):
         super(TestServerAddFixedIP, self).setUp()
 
-        # Get a shortcut to the compute client ServerManager Mock
-        self.networks_mock = self.app.client_manager.compute.networks
-
         # Get the command object to test
         self.cmd = server.AddFixedIP(self.app, None)
 
         # Set add_fixed_ip method to be tested.
         self.methods = {
-            'add_fixed_ip': None,
+            'interface_attach': None,
         }
 
-    def test_server_add_fixed_ip(self):
+    def _test_server_add_fixed_ip(self, extralist, fixed_ip_address):
         servers = self.setup_servers_mock(count=1)
         network = compute_fakes.FakeNetwork.create_one_network()
-        self.networks_mock.get.return_value = network
+        with mock.patch(
+            'openstackclient.api.compute_v2.APIv2.network_find'
+        ) as net_mock:
+            net_mock.return_value = network
 
-        arglist = [
-            servers[0].id,
-            network.id,
-        ]
-        verifylist = [
-            ('server', servers[0].id),
-            ('network', network.id)
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+            arglist = [
+                servers[0].id,
+                network['id'],
+            ] + extralist
+            verifylist = [
+                ('server', servers[0].id),
+                ('network', network['id']),
+                ('fixed_ip_address', fixed_ip_address),
+            ]
+            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        result = self.cmd.take_action(parsed_args)
+            result = self.cmd.take_action(parsed_args)
 
-        servers[0].add_fixed_ip.assert_called_once_with(
-            network.id,
-        )
-        self.assertIsNone(result)
+            servers[0].interface_attach.assert_called_once_with(
+                port_id=None,
+                net_id=network['id'],
+                fixed_ip=fixed_ip_address,
+            )
+            self.assertIsNone(result)
+
+    def test_server_add_fixed_ip(self):
+        self._test_server_add_fixed_ip([], None)
+
+    def test_server_add_specific_fixed_ip(self):
+        extralist = ['--fixed-ip-address', '5.6.7.8']
+        self._test_server_add_fixed_ip(extralist, '5.6.7.8')
 
 
 class TestServerAddFloatingIP(TestServer):
@@ -135,9 +140,6 @@ class TestServerAddFloatingIP(TestServer):
     def setUp(self):
         super(TestServerAddFloatingIP, self).setUp()
 
-        # Get a shortcut to the compute client ServerManager Mock
-        self.networks_mock = self.app.client_manager.compute.networks
-
         # Get the command object to test
         self.cmd = server.AddFloatingIP(self.app, None)
 
@@ -146,25 +148,87 @@ class TestServerAddFloatingIP(TestServer):
             'add_floating_ip': None,
         }
 
-    def test_server_add_floating_ip(self):
+        self.find_port = mock.Mock()
+        self.app.client_manager.network.find_port = self.find_port
+
+    def _test_server_add_floating_ip(self, extralist, fixed_ip_address):
         servers = self.setup_servers_mock(count=1)
 
         arglist = [
             servers[0].id,
             '1.2.3.4',
-        ]
+        ] + extralist
         verifylist = [
             ('server', servers[0].id),
             ('ip_address', '1.2.3.4'),
+            ('fixed_ip_address', fixed_ip_address),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         result = self.cmd.take_action(parsed_args)
 
-        servers[0].add_floating_ip.assert_called_once_with('1.2.3.4')
+        servers[0].add_floating_ip.assert_called_once_with('1.2.3.4',
+                                                           fixed_ip_address)
         self.assertIsNone(result)
 
+    def test_server_add_floating_ip(self):
+        self._test_server_add_floating_ip([], None)
 
+    def test_server_add_floating_ip_to_fixed_ip(self):
+        extralist = ['--fixed-ip-address', '5.6.7.8']
+        self._test_server_add_floating_ip(extralist, '5.6.7.8')
+
+
+class TestServerAddPort(TestServer):
+
+    def setUp(self):
+        super(TestServerAddPort, self).setUp()
+
+        # Get the command object to test
+        self.cmd = server.AddPort(self.app, None)
+
+        # Set add_fixed_ip method to be tested.
+        self.methods = {
+            'interface_attach': None,
+        }
+
+        self.find_port = mock.Mock()
+        self.app.client_manager.network.find_port = self.find_port
+
+    def _test_server_add_port(self, port_id):
+        servers = self.setup_servers_mock(count=1)
+        port = 'fake-port'
+
+        arglist = [
+            servers[0].id,
+            port,
+        ]
+        verifylist = [
+            ('server', servers[0].id),
+            ('port', port)
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+
+        servers[0].interface_attach.assert_called_once_with(
+            port_id=port_id, net_id=None, fixed_ip=None)
+        self.assertIsNone(result)
+
+    def test_server_add_port(self):
+        self._test_server_add_port(self.find_port.return_value.id)
+        self.find_port.assert_called_once_with(
+            'fake-port', ignore_missing=False)
+
+    def test_server_add_port_no_neutron(self):
+        self.app.client_manager.network_endpoint_enabled = False
+        self._test_server_add_port('fake-port')
+        self.find_port.assert_not_called()
+
+
+@mock.patch(
+    'openstackclient.api.compute_v2.APIv2.security_group_find'
+)
 class TestServerAddSecurityGroup(TestServer):
 
     def setUp(self):
@@ -172,11 +236,9 @@ class TestServerAddSecurityGroup(TestServer):
 
         self.security_group = \
             compute_fakes.FakeSecurityGroup.create_one_security_group()
-        # This is the return value for utils.find_resource() for security group
-        self.security_groups_mock.get.return_value = self.security_group
 
         attrs = {
-            'security_groups': [{'name': self.security_group.id}]
+            'security_groups': [{'name': self.security_group['id']}]
         }
         methods = {
             'add_security_group': None,
@@ -192,23 +254,24 @@ class TestServerAddSecurityGroup(TestServer):
         # Get the command object to test
         self.cmd = server.AddServerSecurityGroup(self.app, None)
 
-    def test_server_add_security_group(self):
+    def test_server_add_security_group(self, sg_find_mock):
+        sg_find_mock.return_value = self.security_group
         arglist = [
             self.server.id,
-            self.security_group.id
+            self.security_group['id']
         ]
         verifylist = [
             ('server', self.server.id),
-            ('group', self.security_group.id),
+            ('group', self.security_group['id']),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         result = self.cmd.take_action(parsed_args)
-        self.security_groups_mock.get.assert_called_with(
-            self.security_group.id,
+        sg_find_mock.assert_called_with(
+            self.security_group['id'],
         )
         self.servers_mock.get.assert_called_with(self.server.id)
         self.server.add_security_group.assert_called_with(
-            self.security_group.id,
+            self.security_group['id'],
         )
         self.assertIsNone(result)
 
@@ -384,14 +447,18 @@ class TestServerCreate(TestServer):
         arglist = [
             '--image', 'image1',
             '--flavor', 'flavor1',
-            '--nic', 'net-id=net1',
-            '--nic', 'port-id=port1',
+            '--network', 'net1',
+            '--nic', 'net-id=net1,v4-fixed-ip=10.0.0.2',
+            '--port', 'port1',
+            '--network', 'net1',
+            '--nic', 'port-id=port2',
             self.new_server.name,
         ]
         verifylist = [
             ('image', 'image1'),
             ('flavor', 'flavor1'),
-            ('nic', ['net-id=net1', 'port-id=port1']),
+            ('nic', ['net-id=net1', 'net-id=net1,v4-fixed-ip=10.0.0.2',
+                     'port-id=port1', 'net-id=net1', 'port-id=port2']),
             ('config_drive', False),
             ('server_name', self.new_server.name),
         ]
@@ -409,22 +476,24 @@ class TestServerCreate(TestServer):
         network_client = self.app.client_manager.network
         network_client.find_network = find_network
         network_client.find_port = find_port
-        network_resource = mock.Mock()
-        network_resource.id = 'net1_uuid'
-        port_resource = mock.Mock()
-        port_resource.id = 'port1_uuid'
+        network_resource = mock.Mock(id='net1_uuid')
+        port1_resource = mock.Mock(id='port1_uuid')
+        port2_resource = mock.Mock(id='port2_uuid')
         find_network.return_value = network_resource
-        find_port.return_value = port_resource
+        find_port.side_effect = (lambda port_id, ignore_missing:
+                                 {"port1": port1_resource,
+                                  "port2": port2_resource}[port_id])
 
         # Mock sdk APIs.
-        _network = mock.Mock()
-        _network.id = 'net1_uuid'
-        _port = mock.Mock()
-        _port.id = 'port1_uuid'
+        _network = mock.Mock(id='net1_uuid')
+        _port1 = mock.Mock(id='port1_uuid')
+        _port2 = mock.Mock(id='port2_uuid')
         find_network = mock.Mock()
         find_port = mock.Mock()
         find_network.return_value = _network
-        find_port.return_value = _port
+        find_port.side_effect = (lambda port_id, ignore_missing:
+                                 {"port1": _port1,
+                                  "port2": _port2}[port_id])
         self.app.client_manager.network.find_network = find_network
         self.app.client_manager.network.find_port = find_port
 
@@ -449,10 +518,22 @@ class TestServerCreate(TestServer):
                    'v4-fixed-ip': '',
                    'v6-fixed-ip': '',
                    'port-id': ''},
+                  {'net-id': 'net1_uuid',
+                   'v4-fixed-ip': '10.0.0.2',
+                   'v6-fixed-ip': '',
+                   'port-id': ''},
                   {'net-id': '',
                    'v4-fixed-ip': '',
                    'v6-fixed-ip': '',
-                   'port-id': 'port1_uuid'}],
+                   'port-id': 'port1_uuid'},
+                  {'net-id': 'net1_uuid',
+                   'v4-fixed-ip': '',
+                   'v6-fixed-ip': '',
+                   'port-id': ''},
+                  {'net-id': '',
+                   'v4-fixed-ip': '',
+                   'v6-fixed-ip': '',
+                   'port-id': 'port2_uuid'}],
             scheduler_hints={},
             config_drive=None,
         )
@@ -585,8 +666,7 @@ class TestServerCreate(TestServer):
         find_port = mock.Mock()
         network_client = self.app.client_manager.network
         network_client.find_port = find_port
-        port_resource = mock.Mock()
-        port_resource.id = 'port1_uuid'
+        port_resource = mock.Mock(id='port1_uuid')
         find_port.return_value = port_resource
 
         self.assertRaises(exceptions.CommandError,
@@ -808,8 +888,7 @@ class TestServerCreate(TestServer):
                 'uuid': real_volume_mapping.split(':', 1)[0],
                 'destination_type': 'volume',
                 'source_type': 'volume',
-                'delete_on_termination': '0',
-                'volume_size': ''
+                'delete_on_termination': '0'
             }],
             nics=[],
             scheduler_hints={},
@@ -957,6 +1036,8 @@ class TestServerList(TestServer):
         'Networks',
         'Image Name',
         'Image ID',
+        'Flavor Name',
+        'Flavor ID',
         'Availability Zone',
         'Host',
         'Properties',
@@ -1027,6 +1108,12 @@ class TestServerList(TestServer):
             for s in self.servers
         ]
 
+        Flavor = collections.namedtuple('Flavor', 'id name')
+        self.flavors_mock.list.return_value = [
+            Flavor(id=s.flavor['id'], name=self.flavor.name)
+            for s in self.servers
+        ]
+
         for s in self.servers:
             self.data.append((
                 s.id,
@@ -1046,6 +1133,8 @@ class TestServerList(TestServer):
                 server._format_servers_list_networks(s.networks),
                 self.image.name,
                 s.image['id'],
+                self.flavor.name,
+                s.flavor['id'],
                 getattr(s, 'OS-EXT-AZ:availability_zone'),
                 getattr(s, 'OS-EXT-SRV-ATTR:host'),
                 s.Metadata,
@@ -1185,6 +1274,209 @@ class TestServerLock(TestServer):
         self.run_method_with_servers('lock', 3)
 
 
+class TestServerMigrate(TestServer):
+
+    def setUp(self):
+        super(TestServerMigrate, self).setUp()
+
+        methods = {
+            'migrate': None,
+            'live_migrate': None,
+        }
+        self.server = compute_fakes.FakeServer.create_one_server(
+            methods=methods)
+
+        # This is the return value for utils.find_resource()
+        self.servers_mock.get.return_value = self.server
+
+        self.servers_mock.migrate.return_value = None
+        self.servers_mock.live_migrate.return_value = None
+
+        # Get the command object to test
+        self.cmd = server.MigrateServer(self.app, None)
+
+    def test_server_migrate_no_options(self):
+        arglist = [
+            self.server.id,
+        ]
+        verifylist = [
+            ('live', None),
+            ('block_migration', False),
+            ('disk_overcommit', False),
+            ('wait', False),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+
+        self.servers_mock.get.assert_called_with(self.server.id)
+        self.server.migrate.assert_called_with()
+        self.assertNotCalled(self.servers_mock.live_migrate)
+        self.assertIsNone(result)
+
+    def test_server_migrate_with_block_migration(self):
+        arglist = [
+            '--block-migration', self.server.id,
+        ]
+        verifylist = [
+            ('live', None),
+            ('block_migration', True),
+            ('disk_overcommit', False),
+            ('wait', False),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.assertRaises(exceptions.CommandError, self.cmd.take_action,
+                          parsed_args)
+
+        self.servers_mock.get.assert_called_with(self.server.id)
+        self.assertNotCalled(self.servers_mock.live_migrate)
+        self.assertNotCalled(self.servers_mock.migrate)
+
+    def test_server_migrate_with_disk_overcommit(self):
+        arglist = [
+            '--disk-overcommit', self.server.id,
+        ]
+        verifylist = [
+            ('live', None),
+            ('block_migration', False),
+            ('disk_overcommit', True),
+            ('wait', False),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.assertRaises(exceptions.CommandError, self.cmd.take_action,
+                          parsed_args)
+
+        self.servers_mock.get.assert_called_with(self.server.id)
+        self.assertNotCalled(self.servers_mock.live_migrate)
+        self.assertNotCalled(self.servers_mock.migrate)
+
+    def test_server_live_migrate(self):
+        arglist = [
+            '--live', 'fakehost', self.server.id,
+        ]
+        verifylist = [
+            ('live', 'fakehost'),
+            ('block_migration', False),
+            ('disk_overcommit', False),
+            ('wait', False),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+
+        self.servers_mock.get.assert_called_with(self.server.id)
+        self.server.live_migrate.assert_called_with(block_migration=False,
+                                                    disk_over_commit=False,
+                                                    host='fakehost')
+        self.assertNotCalled(self.servers_mock.migrate)
+        self.assertIsNone(result)
+
+    def test_server_block_live_migrate(self):
+        arglist = [
+            '--live', 'fakehost', '--block-migration', self.server.id,
+        ]
+        verifylist = [
+            ('live', 'fakehost'),
+            ('block_migration', True),
+            ('disk_overcommit', False),
+            ('wait', False),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+
+        self.servers_mock.get.assert_called_with(self.server.id)
+        self.server.live_migrate.assert_called_with(block_migration=True,
+                                                    disk_over_commit=False,
+                                                    host='fakehost')
+        self.assertNotCalled(self.servers_mock.migrate)
+        self.assertIsNone(result)
+
+    def test_server_live_migrate_with_disk_overcommit(self):
+        arglist = [
+            '--live', 'fakehost', '--disk-overcommit', self.server.id,
+        ]
+        verifylist = [
+            ('live', 'fakehost'),
+            ('block_migration', False),
+            ('disk_overcommit', True),
+            ('wait', False),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+
+        self.servers_mock.get.assert_called_with(self.server.id)
+        self.server.live_migrate.assert_called_with(block_migration=False,
+                                                    disk_over_commit=True,
+                                                    host='fakehost')
+        self.assertNotCalled(self.servers_mock.migrate)
+        self.assertIsNone(result)
+
+    def test_server_live_migrate_with_false_value_options(self):
+        arglist = [
+            '--live', 'fakehost', '--no-disk-overcommit',
+            '--shared-migration', self.server.id,
+        ]
+        verifylist = [
+            ('live', 'fakehost'),
+            ('block_migration', False),
+            ('disk_overcommit', False),
+            ('wait', False),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+
+        self.servers_mock.get.assert_called_with(self.server.id)
+        self.server.live_migrate.assert_called_with(block_migration=False,
+                                                    disk_over_commit=False,
+                                                    host='fakehost')
+        self.assertNotCalled(self.servers_mock.migrate)
+        self.assertIsNone(result)
+
+    @mock.patch.object(common_utils, 'wait_for_status', return_value=True)
+    def test_server_migrate_with_wait(self, mock_wait_for_status):
+        arglist = [
+            '--wait', self.server.id,
+        ]
+        verifylist = [
+            ('live', None),
+            ('block_migration', False),
+            ('disk_overcommit', False),
+            ('wait', True),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+
+        self.servers_mock.get.assert_called_with(self.server.id)
+        self.server.migrate.assert_called_with()
+        self.assertNotCalled(self.servers_mock.live_migrate)
+        self.assertIsNone(result)
+
+    @mock.patch.object(common_utils, 'wait_for_status', return_value=False)
+    def test_server_migrate_with_wait_fails(self, mock_wait_for_status):
+        arglist = [
+            '--wait', self.server.id,
+        ]
+        verifylist = [
+            ('live', None),
+            ('block_migration', False),
+            ('disk_overcommit', False),
+            ('wait', True),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.assertRaises(SystemExit, self.cmd.take_action, parsed_args)
+
+        self.servers_mock.get.assert_called_with(self.server.id)
+        self.server.migrate.assert_called_with()
+        self.assertNotCalled(self.servers_mock.live_migrate)
+
+
 class TestServerPause(TestServer):
 
     def setUp(self):
@@ -1391,6 +1683,55 @@ class TestServerRemoveFloatingIP(TestServer):
         self.assertIsNone(result)
 
 
+class TestServerRemovePort(TestServer):
+
+    def setUp(self):
+        super(TestServerRemovePort, self).setUp()
+
+        # Get the command object to test
+        self.cmd = server.RemovePort(self.app, None)
+
+        # Set method to be tested.
+        self.methods = {
+            'interface_detach': None,
+        }
+
+        self.find_port = mock.Mock()
+        self.app.client_manager.network.find_port = self.find_port
+
+    def _test_server_remove_port(self, port_id):
+        servers = self.setup_servers_mock(count=1)
+        port = 'fake-port'
+
+        arglist = [
+            servers[0].id,
+            port,
+        ]
+        verifylist = [
+            ('server', servers[0].id),
+            ('port', port),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+
+        servers[0].interface_detach.assert_called_once_with(port_id)
+        self.assertIsNone(result)
+
+    def test_server_remove_port(self):
+        self._test_server_remove_port(self.find_port.return_value.id)
+        self.find_port.assert_called_once_with(
+            'fake-port', ignore_missing=False)
+
+    def test_server_remove_port_no_neutron(self):
+        self.app.client_manager.network_endpoint_enabled = False
+        self._test_server_remove_port('fake-port')
+        self.find_port.assert_not_called()
+
+
+@mock.patch(
+    'openstackclient.api.compute_v2.APIv2.security_group_find'
+)
 class TestServerRemoveSecurityGroup(TestServer):
 
     def setUp(self):
@@ -1398,11 +1739,9 @@ class TestServerRemoveSecurityGroup(TestServer):
 
         self.security_group = \
             compute_fakes.FakeSecurityGroup.create_one_security_group()
-        # This is the return value for utils.find_resource() for security group
-        self.security_groups_mock.get.return_value = self.security_group
 
         attrs = {
-            'security_groups': [{'name': self.security_group.id}]
+            'security_groups': [{'name': self.security_group['id']}]
         }
         methods = {
             'remove_security_group': None,
@@ -1418,23 +1757,24 @@ class TestServerRemoveSecurityGroup(TestServer):
         # Get the command object to test
         self.cmd = server.RemoveServerSecurityGroup(self.app, None)
 
-    def test_server_remove_security_group(self):
+    def test_server_remove_security_group(self, sg_find_mock):
+        sg_find_mock.return_value = self.security_group
         arglist = [
             self.server.id,
-            self.security_group.id
+            self.security_group['id']
         ]
         verifylist = [
             ('server', self.server.id),
-            ('group', self.security_group.id),
+            ('group', self.security_group['id']),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         result = self.cmd.take_action(parsed_args)
-        self.security_groups_mock.get.assert_called_with(
-            self.security_group.id,
+        sg_find_mock.assert_called_with(
+            self.security_group['id'],
         )
         self.servers_mock.get.assert_called_with(self.server.id)
         self.server.remove_security_group.assert_called_with(
-            self.security_group.id,
+            self.security_group['id'],
         )
         self.assertIsNone(result)
 
