@@ -71,6 +71,10 @@ class TestNetworkV2(utils.TestCommand):
             token=fakes.AUTH_TOKEN,
         )
 
+        self.app.client_manager.sdk_connection = mock.Mock()
+        self.app.client_manager.sdk_connection.network = \
+            self.app.client_manager.network
+
         self.app.client_manager.identity = (
             identity_fakes_v3.FakeIdentityv3Client(
                 endpoint=fakes.AUTH_URL,
@@ -153,6 +157,31 @@ class FakeAddressScope(object):
         return mock.Mock(side_effect=address_scopes)
 
 
+class FakeAutoAllocatedTopology(object):
+    """Fake Auto Allocated Topology"""
+
+    @staticmethod
+    def create_one_topology(attrs=None):
+        attrs = attrs or {}
+
+        auto_allocated_topology_attrs = {
+            'id': 'network-id-' + uuid.uuid4().hex,
+            'tenant_id': 'project-id-' + uuid.uuid4().hex,
+        }
+
+        auto_allocated_topology_attrs.update(attrs)
+
+        auto_allocated_topology = fakes.FakeResource(
+            info=copy.deepcopy(auto_allocated_topology_attrs),
+            loaded=True)
+
+        auto_allocated_topology.project_id = auto_allocated_topology_attrs[
+            'tenant_id'
+        ]
+
+        return auto_allocated_topology
+
+
 class FakeAvailabilityZone(object):
     """Fake one or more network availability zones (AZs)."""
 
@@ -386,6 +415,69 @@ class FakeNetwork(object):
         return mock.Mock(side_effect=networks)
 
 
+class FakeNetworkFlavor(object):
+    """Fake Network Flavor."""
+
+    @staticmethod
+    def create_one_network_flavor(attrs=None):
+        """Create a fake network flavor.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :return:
+            A FakeResource object faking the network flavor
+        """
+        attrs = attrs or {}
+
+        fake_uuid = uuid.uuid4().hex
+        network_flavor_attrs = {
+            'description': 'network-flavor-description-' + fake_uuid,
+            'enabled': True,
+            'id': 'network-flavor-id-' + fake_uuid,
+            'name': 'network-flavor-name-' + fake_uuid,
+            'service_type': 'vpn',
+            'tenant_id': 'project-id-' + uuid.uuid4().hex,
+        }
+
+        # Overwrite default attributes.
+        network_flavor_attrs.update(attrs)
+
+        network_flavor = fakes.FakeResource(
+            info=copy.deepcopy(network_flavor_attrs),
+            loaded=True
+        )
+
+        network_flavor.project_id = network_flavor_attrs['tenant_id']
+        network_flavor.is_enabled = network_flavor_attrs['enabled']
+
+        return network_flavor
+
+    @staticmethod
+    def create_flavor(attrs=None, count=2):
+        """Create multiple fake network flavors.
+
+        :param Dictionary attrs:
+            A dictionary with all attributes
+        :param int count:
+            The number of network flavors to fake
+        :return:
+            A list of FakeResource objects faking the network falvors
+        """
+        network_flavors = []
+        for i in range(0, count):
+            network_flavors.append(
+                FakeNetworkFlavor.create_one_network_flavor(attrs)
+            )
+        return network_flavors
+
+    @staticmethod
+    def get_flavor(network_flavors=None, count=2):
+        """Get a list of flavors."""
+        if network_flavors is None:
+            network_flavors = (FakeNetworkFlavor.create_flavor(count))
+        return mock.Mock(side_effect=network_flavors)
+
+
 class FakeNetworkSegment(object):
     """Fake one or more network segments."""
 
@@ -470,15 +562,17 @@ class FakePort(object):
             'dns_assignment': [{}],
             'dns_name': 'dns-name-' + uuid.uuid4().hex,
             'extra_dhcp_opts': [{}],
-            'fixed_ips': [{}],
+            'fixed_ips': [{'ip_address': '10.0.0.3',
+                           'subnet_id': 'subnet-id-' + uuid.uuid4().hex}],
             'id': 'port-id-' + uuid.uuid4().hex,
             'mac_address': 'fa:16:3e:a9:4e:72',
             'name': 'port-name-' + uuid.uuid4().hex,
             'network_id': 'network-id-' + uuid.uuid4().hex,
             'port_security_enabled': True,
-            'security_groups': [],
+            'security_group_ids': [],
             'status': 'ACTIVE',
             'tenant_id': 'project-id-' + uuid.uuid4().hex,
+            'qos_policy_id': 'qos-policy-id-' + uuid.uuid4().hex,
         }
 
         # Overwrite default attributes.
@@ -496,7 +590,8 @@ class FakePort(object):
         port.is_admin_state_up = port_attrs['admin_state_up']
         port.is_port_security_enabled = port_attrs['port_security_enabled']
         port.project_id = port_attrs['tenant_id']
-        port.security_group_ids = port_attrs['security_groups']
+        port.security_group_ids = port_attrs['security_group_ids']
+        port.qos_policy_id = port_attrs['qos_policy_id']
 
         return port
 
@@ -676,6 +771,53 @@ class FakeNetworkRBAC(object):
         return mock.Mock(side_effect=rbac_policies)
 
 
+class FakeNetworkFlavorProfile(object):
+    """Fake network flavor profile."""
+
+    @staticmethod
+    def create_one_service_profile(attrs=None):
+        """Create flavor profile."""
+        attrs = attrs or {}
+
+        flavor_profile_attrs = {
+            'id': 'flavor-profile-id' + uuid.uuid4().hex,
+            'description': 'flavor-profile-description-' + uuid.uuid4().hex,
+            'tenant_id': 'project-id-' + uuid.uuid4().hex,
+            'driver': 'driver-' + uuid.uuid4().hex,
+            'metainfo': 'metainfo-' + uuid.uuid4().hex,
+            'enabled': True
+        }
+
+        flavor_profile_attrs.update(attrs)
+
+        flavor_profile = fakes.FakeResource(
+            info=copy.deepcopy(flavor_profile_attrs),
+            loaded=True)
+
+        flavor_profile.project_id = flavor_profile_attrs['tenant_id']
+        flavor_profile.is_enabled = flavor_profile_attrs['enabled']
+
+        return flavor_profile
+
+    @staticmethod
+    def create_service_profile(attrs=None, count=2):
+        """Create multiple flavor profiles."""
+
+        flavor_profiles = []
+        for i in range(0, count):
+            flavor_profiles.append(FakeNetworkFlavorProfile.
+                                   create_one_service_profile(attrs))
+        return flavor_profiles
+
+    @staticmethod
+    def get_service_profile(flavor_profile=None, count=2):
+        """Get a list of flavor profiles."""
+        if flavor_profile is None:
+            flavor_profile = (FakeNetworkFlavorProfile.
+                              create_service_profile(count))
+        return mock.Mock(side_effect=flavor_profile)
+
+
 class FakeNetworkQosPolicy(object):
     """Fake one or more QoS policies."""
 
@@ -978,7 +1120,7 @@ class FakeSecurityGroup(object):
             'id': 'security-group-id-' + uuid.uuid4().hex,
             'name': 'security-group-name-' + uuid.uuid4().hex,
             'description': 'security-group-description-' + uuid.uuid4().hex,
-            'tenant_id': 'project-id-' + uuid.uuid4().hex,
+            'project_id': 'project-id-' + uuid.uuid4().hex,
             'security_group_rules': [],
         }
 
@@ -990,7 +1132,7 @@ class FakeSecurityGroup(object):
             loaded=True)
 
         # Set attributes with special mapping in OpenStack SDK.
-        security_group.project_id = security_group_attrs['tenant_id']
+        security_group.project_id = security_group_attrs['project_id']
 
         return security_group
 
@@ -1051,7 +1193,7 @@ class FakeSecurityGroupRule(object):
             'description': 'security-group-rule-description-' +
                            uuid.uuid4().hex,
             'direction': 'ingress',
-            'ethertype': 'IPv4',
+            'ether_type': 'IPv4',
             'id': 'security-group-rule-id-' + uuid.uuid4().hex,
             'port_range_max': None,
             'port_range_min': None,
@@ -1489,3 +1631,53 @@ class FakeNetworkServiceProvider(object):
                                      create_one_network_service_provider(
                                          attrs))
         return service_providers
+
+
+class FakeQuota(object):
+    """Fake quota"""
+
+    @staticmethod
+    def create_one_net_quota(attrs=None):
+        """Create one quota"""
+        attrs = attrs or {}
+
+        quota_attrs = {
+            'floating_ips': 20,
+            'networks': 25,
+            'ports': 11,
+            'rbac_policies': 15,
+            'routers': 40,
+            'security_groups': 10,
+            'security_group_rules': 100,
+            'subnets': 20,
+            'subnet_pools': 30}
+
+        quota_attrs.update(attrs)
+
+        quota = fakes.FakeResource(
+            info=copy.deepcopy(quota_attrs),
+            loaded=True)
+        return quota
+
+    @staticmethod
+    def create_one_default_net_quota(attrs=None):
+        """Create one quota"""
+        attrs = attrs or {}
+
+        quota_attrs = {
+            'floatingip': 30,
+            'network': 20,
+            'port': 10,
+            'rbac_policy': 25,
+            'router': 30,
+            'security_group': 30,
+            'security_group_rule': 200,
+            'subnet': 10,
+            'subnetpool': 20}
+
+        quota_attrs.update(attrs)
+
+        quota = fakes.FakeResource(
+            info=copy.deepcopy(quota_attrs),
+            loaded=True)
+        return quota
