@@ -14,15 +14,14 @@
 #
 
 import argparse
-import copy
 import mock
 from mock import call
 
 from osc_lib import exceptions
 from osc_lib import utils
 
-from openstackclient.tests.unit import fakes
 from openstackclient.tests.unit.identity.v2_0 import fakes as identity_fakes
+from openstackclient.tests.unit.image.v1 import fakes as image_fakes
 from openstackclient.tests.unit import utils as tests_utils
 from openstackclient.tests.unit.volume.v1 import fakes as volume_fakes
 from openstackclient.volume.v1 import volume
@@ -58,10 +57,6 @@ class TestVolume(volume_fakes.TestVolumev1):
         return volumes
 
 
-# TODO(dtroyer): The volume create tests are incomplete, only the minimal
-#                options and the options that require additional processing
-#                are implemented at this time.
-
 class TestVolumeCreate(TestVolume):
 
     project = identity_fakes.FakeProject.create_one_project()
@@ -321,19 +316,16 @@ class TestVolumeCreate(TestVolume):
         self.assertEqual(self.datalist, data)
 
     def test_volume_create_image_id(self):
-        self.images_mock.get.return_value = fakes.FakeResource(
-            None,
-            copy.deepcopy(volume_fakes.IMAGE),
-            loaded=True,
-        )
+        image = image_fakes.FakeImage.create_one_image()
+        self.images_mock.get.return_value = image
 
         arglist = [
-            '--image', volume_fakes.image_id,
+            '--image', image.id,
             '--size', str(self.new_volume.size),
             self.new_volume.display_name,
         ]
         verifylist = [
-            ('image', volume_fakes.image_id),
+            ('image', image.id),
             ('size', self.new_volume.size),
             ('name', self.new_volume.display_name),
         ]
@@ -360,26 +352,23 @@ class TestVolumeCreate(TestVolume):
             None,
             None,
             None,
-            volume_fakes.image_id,
+            image.id,
         )
 
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.datalist, data)
 
     def test_volume_create_image_name(self):
-        self.images_mock.get.return_value = fakes.FakeResource(
-            None,
-            copy.deepcopy(volume_fakes.IMAGE),
-            loaded=True,
-        )
+        image = image_fakes.FakeImage.create_one_image()
+        self.images_mock.get.return_value = image
 
         arglist = [
-            '--image', volume_fakes.image_name,
+            '--image', image.name,
             '--size', str(self.new_volume.size),
             self.new_volume.display_name,
         ]
         verifylist = [
-            ('image', volume_fakes.image_name),
+            ('image', image.name),
             ('size', self.new_volume.size),
             ('name', self.new_volume.display_name),
         ]
@@ -406,7 +395,7 @@ class TestVolumeCreate(TestVolume):
             None,
             None,
             None,
-            volume_fakes.image_id,
+            image.id,
         )
 
         self.assertEqual(self.columns, columns)
@@ -442,6 +431,142 @@ class TestVolumeCreate(TestVolume):
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.datalist, data)
 
+    def test_volume_create_with_bootable_and_readonly(self):
+        arglist = [
+            '--bootable',
+            '--read-only',
+            '--size', str(self.new_volume.size),
+            self.new_volume.display_name,
+        ]
+        verifylist = [
+            ('bootable', True),
+            ('non_bootable', False),
+            ('read_only', True),
+            ('read_write', False),
+            ('size', self.new_volume.size),
+            ('name', self.new_volume.display_name),
+        ]
+
+        parsed_args = self.check_parser(
+            self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.volumes_mock.create.assert_called_with(
+            self.new_volume.size,
+            None,
+            None,
+            self.new_volume.display_name,
+            None,
+            None,
+            None,
+            None,
+            None,
+            None,
+            None,
+        )
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
+        self.volumes_mock.set_bootable.assert_called_with(
+            self.new_volume.id, True)
+        self.volumes_mock.update_readonly_flag.assert_called_with(
+            self.new_volume.id, True)
+
+    def test_volume_create_with_nonbootable_and_readwrite(self):
+        arglist = [
+            '--non-bootable',
+            '--read-write',
+            '--size', str(self.new_volume.size),
+            self.new_volume.display_name,
+        ]
+        verifylist = [
+            ('bootable', False),
+            ('non_bootable', True),
+            ('read_only', False),
+            ('read_write', True),
+            ('size', self.new_volume.size),
+            ('name', self.new_volume.display_name),
+        ]
+
+        parsed_args = self.check_parser(
+            self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.volumes_mock.create.assert_called_with(
+            self.new_volume.size,
+            None,
+            None,
+            self.new_volume.display_name,
+            None,
+            None,
+            None,
+            None,
+            None,
+            None,
+            None,
+        )
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
+        self.volumes_mock.set_bootable.assert_called_with(
+            self.new_volume.id, False)
+        self.volumes_mock.update_readonly_flag.assert_called_with(
+            self.new_volume.id, False)
+
+    @mock.patch.object(volume.LOG, 'error')
+    def test_volume_create_with_bootable_and_readonly_fail(
+            self, mock_error):
+
+        self.volumes_mock.set_bootable.side_effect = (
+            exceptions.CommandError())
+
+        self.volumes_mock.update_readonly_flag.side_effect = (
+            exceptions.CommandError())
+
+        arglist = [
+            '--bootable',
+            '--read-only',
+            '--size', str(self.new_volume.size),
+            self.new_volume.display_name,
+        ]
+        verifylist = [
+            ('bootable', True),
+            ('non_bootable', False),
+            ('read_only', True),
+            ('read_write', False),
+            ('size', self.new_volume.size),
+            ('name', self.new_volume.display_name),
+        ]
+
+        parsed_args = self.check_parser(
+            self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.volumes_mock.create.assert_called_with(
+            self.new_volume.size,
+            None,
+            None,
+            self.new_volume.display_name,
+            None,
+            None,
+            None,
+            None,
+            None,
+            None,
+            None,
+        )
+
+        self.assertEqual(2, mock_error.call_count)
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(self.datalist, data)
+        self.volumes_mock.set_bootable.assert_called_with(
+            self.new_volume.id, True)
+        self.volumes_mock.update_readonly_flag.assert_called_with(
+            self.new_volume.id, True)
+
     def test_volume_create_without_size(self):
         arglist = [
             self.new_volume.display_name,
@@ -739,6 +864,68 @@ class TestVolumeList(TestVolume):
                           self.cmd, arglist, verifylist)
 
 
+class TestVolumeMigrate(TestVolume):
+
+    _volume = volume_fakes.FakeVolume.create_one_volume()
+
+    def setUp(self):
+        super(TestVolumeMigrate, self).setUp()
+
+        self.volumes_mock.get.return_value = self._volume
+        self.volumes_mock.migrate_volume.return_value = None
+        # Get the command object to test
+        self.cmd = volume.MigrateVolume(self.app, None)
+
+    def test_volume_migrate(self):
+        arglist = [
+            "--host", "host@backend-name#pool",
+            self._volume.id,
+        ]
+        verifylist = [
+            ("force_host_copy", False),
+            ("host", "host@backend-name#pool"),
+            ("volume", self._volume.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.volumes_mock.get.assert_called_once_with(self._volume.id)
+        self.volumes_mock.migrate_volume.assert_called_once_with(
+            self._volume.id, "host@backend-name#pool", False)
+        self.assertIsNone(result)
+
+    def test_volume_migrate_with_option(self):
+        arglist = [
+            "--force-host-copy",
+            "--host", "host@backend-name#pool",
+            self._volume.id,
+        ]
+        verifylist = [
+            ("force_host_copy", True),
+            ("host", "host@backend-name#pool"),
+            ("volume", self._volume.id),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.volumes_mock.get.assert_called_once_with(self._volume.id)
+        self.volumes_mock.migrate_volume.assert_called_once_with(
+            self._volume.id, "host@backend-name#pool", True)
+        self.assertIsNone(result)
+
+    def test_volume_migrate_without_host(self):
+        arglist = [
+            self._volume.id,
+        ]
+        verifylist = [
+            ("force_host_copy", False),
+            ("volume", self._volume.id),
+        ]
+
+        self.assertRaises(tests_utils.ParserException, self.check_parser,
+                          self.cmd, arglist, verifylist)
+
+
 class TestVolumeSet(TestVolume):
 
     _volume = volume_fakes.FakeVolume.create_one_volume()
@@ -844,8 +1031,7 @@ class TestVolumeSet(TestVolume):
         )
         self.assertIsNone(result)
 
-    @mock.patch.object(volume.LOG, 'error')
-    def test_volume_set_size_smaller(self, mock_log_error):
+    def test_volume_set_size_smaller(self):
         self._volume.status = 'available'
         arglist = [
             '--size', '1',
@@ -860,15 +1046,11 @@ class TestVolumeSet(TestVolume):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        result = self.cmd.take_action(parsed_args)
+        self.assertRaises(exceptions.CommandError,
+                          self.cmd.take_action,
+                          parsed_args)
 
-        mock_log_error.assert_called_with("New size must be greater "
-                                          "than %s GB",
-                                          self._volume.size)
-        self.assertIsNone(result)
-
-    @mock.patch.object(volume.LOG, 'error')
-    def test_volume_set_size_not_available(self, mock_log_error):
+    def test_volume_set_size_not_available(self):
         self._volume.status = 'error'
         arglist = [
             '--size', '130',
@@ -883,22 +1065,23 @@ class TestVolumeSet(TestVolume):
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
-        result = self.cmd.take_action(parsed_args)
-
-        mock_log_error.assert_called_with("Volume is in %s state, it must be "
-                                          "available before size can be "
-                                          "extended", 'error')
-        self.assertIsNone(result)
+        self.assertRaises(exceptions.CommandError,
+                          self.cmd.take_action,
+                          parsed_args)
 
     def test_volume_set_property(self):
         arglist = [
+            '--no-property',
             '--property', 'myprop=myvalue',
             self._volume.display_name,
         ]
         verifylist = [
+            ('read_only', False),
+            ('read_write', False),
             ('name', None),
             ('description', None),
             ('size', None),
+            ('no_property', True),
             ('property', {'myprop': 'myvalue'}),
             ('volume', self._volume.display_name),
             ('bootable', False),
@@ -916,6 +1099,11 @@ class TestVolumeSet(TestVolume):
             self._volume.id,
             metadata
         )
+        self.volumes_mock.delete_metadata.assert_called_with(
+            self._volume.id,
+            self._volume.metadata.keys()
+        )
+        self.volumes_mock.update_readonly_flag.assert_not_called()
         self.assertIsNone(result)
 
     def test_volume_set_bootable(self):
@@ -943,6 +1131,44 @@ class TestVolumeSet(TestVolume):
             self.volumes_mock.set_bootable.assert_called_with(
                 self._volume.id, verifylist[index][0][1])
 
+    def test_volume_set_readonly(self):
+        arglist = [
+            '--read-only',
+            self._volume.id
+        ]
+        verifylist = [
+            ('read_only', True),
+            ('read_write', False),
+            ('volume', self._volume.id)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.volumes_mock.update_readonly_flag.assert_called_once_with(
+            self._volume.id,
+            True)
+        self.assertIsNone(result)
+
+    def test_volume_set_read_write(self):
+        arglist = [
+            '--read-write',
+            self._volume.id
+        ]
+        verifylist = [
+            ('read_only', False),
+            ('read_write', True),
+            ('volume', self._volume.id)
+        ]
+
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+        self.volumes_mock.update_readonly_flag.assert_called_once_with(
+            self._volume.id,
+            False)
+        self.assertIsNone(result)
+
 
 class TestVolumeShow(TestVolume):
 
